#!/usr/bin/perl
## ######### PROJECT NAME : ##########
## 
## Lib_CGI.pm for wishmin
## 
## ######### PROJECT DESCRIPTION : ###
## 
## This file provides helpers on CGI and CGI::SESSION.
##
## This file became from an personal homemade project
## named bambooweb (ryzome.info). This file is
## primarily release under the BSD license.
##
## 
## ###################################
## 
## Made by Boutonnet Alexandre
## Email   <aboutonnet@intellique.com>
## 
## Started on  Wed May 13 17:23:41 2009 Boutonnet Alexandre
## Last update Wed May 13 17:25:41 2009 Boutonnet Alexandre
## 
## ###################################
##

package Lib_CGI;

use CGI;
use CGI::Session;
use CGI::Cookie;

# Cette variable contient le nom du cookie
my $SESSION_NAME = "Wishmin";


# Declaration de vars globales.. ca pue je sais..
my $cgi;
my $session;
my $cookie;
my @cookies_tab;

# Fonction de creation de l'instance de cgi
# Retourne 0 en cas de succes et affiche le
# header d'urgence en cas d'echec..
sub new_cgi
{
    $cgi = CGI->new();
    
    $| = 1;
    
    send_critical_header("CGI new failed..") if (!defined($cgi));
    return 0;
}

sub print_html_header
{
    if (defined($cgi))
    {
	if (@cookies_tab)
	{
	    print $cgi->header(-type=>'text/html',
			       -charset=>'utf-8',
			       -cookie=>\@cookies_tab);
	}
	else
	{
	    print $cgi->header(-type=>'text/html',
			       -charset=>'utf-8');
	}
	return 0;
    }
    
    # J'ai une erreur de header...
    send_critical_header("CGI ERROR");
}

# Fonction de test si la session en cours est valide ou pas..
# Retourne 0 si ok
# Retourne 1 si non
sub test_session_validity
{
    my $_session = _load_session();
    
    return 1 if ($_session->is_expired);
    return 0;
}

# Fonction de test si la session est vide
# Retourne 0 si ok
# Retourne 1 si non
sub test_session_empty
{
    my $_session = _load_session();
    
    return 1 if ($_session->is_empty);
    return 0;
}

# Fonction de test si le visiteur est logge
# retourne 0 si oui
# retourne 1 si jamais
# retourne 2 si expired
sub test_is_logged
{
    return 2 if (test_session_validity());
    return 1 if (test_session_empty());
    return 0;
}

# Fonction de recuperation d'un parametre..
# Prend en parametre le nom du param..
# retourne le parametre ou undef
sub get_param
{
    my $param = shift;
    
    return $cgi->param($param);
}

# Fonction de creation d'une session..
sub new_session
{
    if (!defined($session))
    {
	CGI::Session->name($SESSION_NAME);
	  $session = new CGI::Session(); #undef, undef, 
#			{Directory=>'/tmp'});
	  return 1 if (!defined($session));
	  
	  $session->expire("~logged-in", "20m");
	  
	  my $_cookie = $cgi->cookie(-name => $session->name, 
				     -value  => $session->id );
	  push(@cookies_tab, $_cookie);
	  
	  return 0;
      }
    return 1;
}

# Fonction de suppression d'une session..
sub close_session
{
    $session =_load_session() if (!defined($session));
    $session->delete();
    return 0;
}

# Fonction de set d'un param a la session
sub session_set_param
{
    my ($var, $value) = @_;
    if (defined($session))
    {
	$session->param($var, $value);
	return 0;
    }
    return 1;
}

# Fonction de recuperation d'une variable dans la session
sub session_get_param
{
    my $var = shift;

    if (defined($session))
    {
	return $session->param($var);
    }
    else
    {
	my $_session = _load_session();
	return $_session->param($var);
    }
    return 1;
}

# Fonction de recuperation de l'id de la session
sub get_session_id
{
    if (defined($session))
    {
	return $session->id();	
    }
    my $_session = CGI::Session->load()
	or send_critical_header(CGI::Session->errstr);
    return $_session->id();
}

# Fonction de recuperation du remote host
sub get_remote_host
{
    return 1 if (!defined($cgi));
    return $cgi->remote_host();
}

# Fonction de recuperation du remote_host stocke dans une session
sub get_loaded_remote_host
{
    my $_session = CGI::Session->load()
	or send_critical_header(CGI::Session->errstr);
    
    return $_session->remote_addr();
}

# Fonction d'envoie d'un header d'urgence..
# Prend en param un message d'erreur
sub send_critical_header
{
    my $msg = shift;
    
    print <<EOF;
Content-Type: text/html

<html>
<head>
<title>BambooWed Critical Error</title>
</head>
<body>
<center><h1>Error !!</h1></center>
<pre>$msg</pre>
</body>
</html>
EOF
	exit 0;
}

# Fonction de debug... print Dumper
sub debug
{
    use Data::Dumper;
    print "<pre>";
    print Dumper $cgi;
    print "-----------------------\n";
    print Dumper $session;
    print "</pre>";
}

# Fonction de load d'un session
sub _load_session
{
    CGI::Session->name($SESSION_NAME);
      my $_session = CGI::Session->load()
	  or send_critical_header(CGI::Session->errstr);
      
      return($_session);
}

1;
